@@ -49,7 +49,6 @@ init_appearance_data (int *argc, char ***argv)
   g_thread_init (NULL);
   theme_thumbnail_factory_init (*argc, *argv);
   gtk_init (argc, argv);
-  gnome_vfs_init ();
   activate_settings_daemon ();
 
   /* set up the data */
@@ -112,6 +111,7 @@ main (int argc, char **argv)
         G_OPTION_FLAG_IN_MAIN,
         G_OPTION_ARG_STRING,
         &start_page,
+        /* TRANSLATORS: don't translate the terms in brackets */
         N_("Specify the name of the page to show (theme|background|fonts|interface)"),
         N_("page") },
       { G_OPTION_REMAINING,
@@ -142,9 +142,12 @@ main (int argc, char **argv)
         GNOME_PARAM_GOPTION_CONTEXT, option_context,
         NULL);
 
-  if (install_filename != NULL)
-     gnome_theme_install_from_uri (install_filename, NULL);
-  g_free (install_filename);
+  if (install_filename != NULL) {
+    GFile *inst = g_file_new_for_commandline_arg (install_filename);
+    g_free (install_filename);
+    gnome_theme_install (inst, NULL);
+    g_object_unref (inst);
+  }
 
   /* init tabs */
   themes_init (data);
@@ -156,7 +159,7 @@ main (int argc, char **argv)
 
   /* prepare the main window */
   w = glade_xml_get_widget (data->xml, "appearance_window");
-  capplet_set_icon (w, "gnome-settings-theme");
+  capplet_set_icon (w, "preferences-desktop-theme");
   gtk_widget_show_all (w);
 
   g_signal_connect_after (w, "response",
