@@ -28,11 +28,11 @@
 #include "gtkrc-utils.h"
 #include "gedit-message-area.h"
 #include "wp-cellrenderer.h"
+#include "caption-cellrenderer.h"
 
 #include <glib/gi18n.h>
 #include <libwindow-settings/gnome-wm-manager.h>
 #include <string.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <libgnomeui/gnome-thumbnail.h>
 
 #define CUSTOM_THEME_NAME "__custom__"
@@ -40,7 +40,9 @@
 enum
 {
   RESPONSE_APPLY_BG,
-  RESPONSE_APPLY_FONT
+  RESPONSE_REVERT_FONT,
+  RESPONSE_APPLY_FONT,
+  RESPONSE_INSTALL_ENGINE
 };
 
 enum
@@ -55,6 +57,8 @@ static const GtkTargetEntry drop_types[] =
   {"_NETSCAPE_URL", 0, TARGET_NS_URL}
 };
 
+static void theme_message_area_update (AppearanceData *data);
+
 static time_t
 theme_get_mtime (const char *name)
 {
@@ -63,19 +67,21 @@ theme_get_mtime (const char *name)
 
   theme = gnome_theme_meta_info_find (name);
   if (theme != NULL) {
-    GnomeVFSFileInfo *file_info;
-    GnomeVFSResult result;
+    GFile *file;
+    GFileInfo *file_info;
 
-    file_info = gnome_vfs_file_info_new ();
+    file = g_file_new_for_path (theme->path);
+    file_info = g_file_query_info (file,
+                                   G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                                   G_FILE_QUERY_INFO_NONE,
+                                   NULL, NULL);
+    g_object_unref (file);
 
-    result = gnome_vfs_get_file_info (theme->path, file_info,
-				      GNOME_VFS_FILE_INFO_DEFAULT |
-				      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-
-    if (result == GNOME_VFS_OK)
-      mtime = file_info->mtime;
-
-    gnome_vfs_file_info_unref (file_info);
+    if (file_info != NULL) {
+      mtime = g_file_info_get_attribute_uint64 (file_info,
+                                                G_FILE_ATTRIBUTE_TIME_MODIFIED);
+      g_object_unref (file_info);
+    }
   }
 
   return mtime;
@@ -112,8 +118,6 @@ theme_thumbnail_update (GdkPixbuf *pixbuf,
       g_free (path);
     }
   }
-
-  g_object_unref (pixbuf);
 }
 
 static GdkPixbuf *
@@ -157,9 +161,10 @@ theme_thumbnail_generate (GnomeThemeMetaInfo *info, AppearanceData *data)
 
   thumb = theme_get_thumbnail_from_cache (info, data);
 
-  if (thumb != NULL)
+  if (thumb != NULL) {
     theme_thumbnail_update (thumb, info->name, data, FALSE);
-  else
+    g_object_unref (thumb);
+  } else
     generate_meta_theme_thumbnail_async (info,
         (ThemeThumbnailFunc) theme_thumbnail_done_cb, data, NULL);
 }
@@ -243,6 +248,8 @@ theme_load_from_gconf (GConfClient *client)
   if (theme->icon_theme_name == NULL)
     theme->icon_theme_name = g_strdup ("gnome");
 
+  theme->notification_theme_name = gconf_client_get_string (client, NOTIFICATION_THEME_KEY, NULL);
+
   theme->cursor_theme_name = gconf_client_get_string (client, CURSOR_THEME_KEY, NULL);
 #ifdef HAVE_XCURSOR
   theme->cursor_size = gconf_client_get_int (client, CURSOR_SIZE_KEY, NULL);
@@ -416,12 +423,29 @@ theme_set_custom_from_theme (const GnomeThemeMetaInfo *info, AppearanceData *dat
 
 /** GUI Callbacks **/
 
+static void
+custom_font_cb (GtkWidget *button, AppearanceData *data)
+{
+  g_free (data->revert_application_font);
+  g_free (data->revert_documents_font);
+  g_free (data->revert_desktop_font);
+  g_free (data->revert_windowtitle_font);
+  g_free (data->revert_monospace_font);
+  data->revert_application_font = NULL;
+  data->revert_documents_font = NULL;
+  data->revert_desktop_font = NULL;
+  data->revert_windowtitle_font = NULL;
+  data->revert_monospace_font = NULL;
+}
+
 static void
 theme_message_area_response_cb (GtkWidget *w,
                                 gint response_id,
                                 AppearanceData *data)
 {
   const GnomeThemeMetaInfo *theme;
+  gchar *tmpfont;
+  gchar *engine_path;
 
   theme = theme_get_selected (GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list")), data);
   if (!theme)
@@ -434,26 +458,128 @@ theme_message_area_response_cb (GtkWidget *w,
                                theme->background_image, NULL);
       break;
 
+    case RESPONSE_REVERT_FONT:
+      if (data->revert_application_font != NULL) {
+        gconf_client_set_string (data->client, APPLICATION_FONT_KEY,
+                                 data->revert_application_font, NULL);
+        g_free (data->revert_application_font);
+        data->revert_application_font = NULL;
+      }
+
+      if (data->revert_documents_font != NULL) {
+        gconf_client_set_string (data->client, DOCUMENTS_FONT_KEY,
+                                 data->revert_documents_font, NULL);
+        g_free (data->revert_documents_font);
+        data->revert_documents_font = NULL;
+      }
+
+      if (data->revert_desktop_font != NULL) {
+        gconf_client_set_string (data->client, DESKTOP_FONT_KEY,
+                                 data->revert_desktop_font, NULL);
+        g_free (data->revert_desktop_font);
+        data->revert_desktop_font = NULL;
+      }
+
+      if (data->revert_windowtitle_font != NULL) {
+        gconf_client_set_string (data->client, WINDOWTITLE_FONT_KEY,
+                                 data->revert_windowtitle_font, NULL);
+        g_free (data->revert_windowtitle_font);
+        data->revert_windowtitle_font = NULL;
+      }
+
+      if (data->revert_monospace_font != NULL) {
+        gconf_client_set_string (data->client, MONOSPACE_FONT_KEY,
+                                 data->revert_monospace_font, NULL);
+        g_free (data->revert_monospace_font);
+        data->revert_monospace_font = NULL;
+      }
+      break;
+
     case RESPONSE_APPLY_FONT:
-      if (theme->application_font)
+      if (theme->application_font) {
+        tmpfont = gconf_client_get_string (data->client, APPLICATION_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_application_font);
+
+          if (strcmp (theme->application_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_application_font = NULL;
+          } else
+            data->revert_application_font = tmpfont;
+        }
         gconf_client_set_string (data->client, APPLICATION_FONT_KEY,
                                  theme->application_font, NULL);
+      }
 
-      if (theme->documents_font)
+      if (theme->documents_font) {
+        tmpfont = gconf_client_get_string (data->client, DOCUMENTS_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_documents_font);
+
+          if (strcmp (theme->documents_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_documents_font = NULL;
+          } else
+            data->revert_documents_font = tmpfont;
+        }
         gconf_client_set_string (data->client, DOCUMENTS_FONT_KEY,
                                  theme->documents_font, NULL);
+      }
 
-      if (theme->desktop_font)
+      if (theme->desktop_font) {
+        tmpfont = gconf_client_get_string (data->client, DESKTOP_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_desktop_font);
+
+          if (strcmp (theme->desktop_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_desktop_font = NULL;
+          } else
+            data->revert_desktop_font = tmpfont;
+        }
         gconf_client_set_string (data->client, DESKTOP_FONT_KEY,
                                  theme->desktop_font, NULL);
+      }
 
-      if (theme->windowtitle_font)
+      if (theme->windowtitle_font) {
+        tmpfont = gconf_client_get_string (data->client, WINDOWTITLE_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_windowtitle_font);
+
+          if (strcmp (theme->windowtitle_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_windowtitle_font = NULL;
+          } else
+            data->revert_windowtitle_font = tmpfont;
+        }
         gconf_client_set_string (data->client, WINDOWTITLE_FONT_KEY,
                                  theme->windowtitle_font, NULL);
+      }
 
-      if (theme->monospace_font)
+      if (theme->monospace_font) {
+        tmpfont = gconf_client_get_string (data->client, MONOSPACE_FONT_KEY, NULL);
+        if (tmpfont != NULL) {
+          g_free (data->revert_monospace_font);
+
+          if (strcmp (theme->monospace_font, tmpfont) == 0) {
+            g_free (tmpfont);
+            data->revert_monospace_font = NULL;
+          } else
+            data->revert_monospace_font = tmpfont;
+        }
         gconf_client_set_string (data->client, MONOSPACE_FONT_KEY,
                                  theme->monospace_font, NULL);
+      }
+      break;
+
+    case RESPONSE_INSTALL_ENGINE:
+      engine_path = gtk_theme_info_missing_engine (theme->gtk_theme_name, FALSE);
+      if (engine_path != NULL) {
+              theme_install_file (GTK_WINDOW (gtk_widget_get_toplevel (data->install_button)),
+                                  engine_path);
+              g_free (engine_path);
+      }
+      theme_message_area_update (data);
       break;
   }
 }
@@ -462,65 +588,79 @@ static void
 theme_message_area_update (AppearanceData *data)
 {
   const GnomeThemeMetaInfo *theme;
-  gboolean show_apply_background = FALSE, show_apply_font = FALSE;
+  gboolean show_apply_background = FALSE;
+  gboolean show_apply_font = FALSE;
+  gboolean show_revert_font = FALSE;
+  gboolean show_error;
   const gchar *message;
   gchar *font;
+  GError *error = NULL;
 
   theme = theme_get_selected (GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list")), data);
 
-  if (!theme)
+  if (!theme) {
+    if (data->theme_message_area != NULL)
+      gtk_widget_hide (data->theme_message_area);
     return;
-
-  if (theme->background_image != NULL) {
-    gchar *background;
-
-    background = gconf_client_get_string (data->client, BACKGROUND_KEY, NULL);
-    show_apply_background =
-        (!background || strcmp (theme->background_image, background) != 0);
-    g_free (background);
   }
 
-  if (theme->application_font) {
-    font = gconf_client_get_string (data->client, APPLICATION_FONT_KEY, NULL);
-    show_apply_font =
-        (!font || strcmp (theme->application_font, font) != 0);
-    g_free (font);
-  }
+  show_error = !gnome_theme_meta_info_validate (theme, &error);
 
-  if (!show_apply_font && theme->documents_font) {
-    font = gconf_client_get_string (data->client, DOCUMENTS_FONT_KEY, NULL);
-    show_apply_font =
-        (!font || strcmp (theme->application_font, font) != 0);
-    g_free (font);
-  }
+  if (!show_error) {
+    if (theme->background_image != NULL) {
+      gchar *background;
 
-  if (!show_apply_font && theme->desktop_font) {
-    font = gconf_client_get_string (data->client, DESKTOP_FONT_KEY, NULL);
-    show_apply_font =
-        (!font || strcmp (theme->application_font, font) != 0);
-    g_free (font);
-  }
+      background = gconf_client_get_string (data->client, BACKGROUND_KEY, NULL);
+      show_apply_background =
+          (!background || strcmp (theme->background_image, background) != 0);
+      g_free (background);
+    }
 
- if (!show_apply_font && theme->windowtitle_font) {
-    font = gconf_client_get_string (data->client, WINDOWTITLE_FONT_KEY, NULL);
-    show_apply_font =
-        (!font || strcmp (theme->application_font, font) != 0);
-    g_free (font);
-  }
+    if (theme->application_font) {
+      font = gconf_client_get_string (data->client, APPLICATION_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
 
-  if (!show_apply_font && theme->monospace_font) {
-    font = gconf_client_get_string (data->client, MONOSPACE_FONT_KEY, NULL);
-    show_apply_font =
-        (!font || strcmp (theme->application_font, font) != 0);
-    g_free (font);
+    if (!show_apply_font && theme->documents_font) {
+      font = gconf_client_get_string (data->client, DOCUMENTS_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
+
+    if (!show_apply_font && theme->desktop_font) {
+      font = gconf_client_get_string (data->client, DESKTOP_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
+
+    if (!show_apply_font && theme->windowtitle_font) {
+      font = gconf_client_get_string (data->client, WINDOWTITLE_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
+
+    if (!show_apply_font && theme->monospace_font) {
+      font = gconf_client_get_string (data->client, MONOSPACE_FONT_KEY, NULL);
+      show_apply_font =
+          (!font || strcmp (theme->application_font, font) != 0);
+      g_free (font);
+    }
+
+    show_revert_font = (data->revert_application_font != NULL ||
+      data->revert_documents_font != NULL || data->revert_desktop_font != NULL ||
+      data->revert_windowtitle_font != NULL || data->revert_monospace_font != NULL);
   }
 
   if (data->theme_message_area == NULL) {
     GtkWidget *hbox;
-    GtkWidget *icon;
     GtkWidget *parent;
 
-    if (!show_apply_background && !show_apply_font)
+    if (!show_apply_background && !show_revert_font && !show_apply_font && !show_error)
       return;
 
     data->theme_message_area = gedit_message_area_new ();
@@ -537,6 +677,14 @@ theme_message_area_update (AppearanceData *data)
         GEDIT_MESSAGE_AREA (data->theme_message_area),
         _("Apply Font"),
         RESPONSE_APPLY_FONT);
+    data->revert_font_button = gedit_message_area_add_button (
+        GEDIT_MESSAGE_AREA (data->theme_message_area),
+        _("Revert Font"),
+        RESPONSE_REVERT_FONT);
+    data->install_button = gedit_message_area_add_button (
+        GEDIT_MESSAGE_AREA (data->theme_message_area),
+        _("Install"),
+        RESPONSE_INSTALL_ENGINE);
 
     data->theme_message_label = gtk_label_new (NULL);
     gtk_widget_show (data->theme_message_label);
@@ -545,10 +693,12 @@ theme_message_area_update (AppearanceData *data)
 
     hbox = gtk_hbox_new (FALSE, 9);
     gtk_widget_show (hbox);
-    icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
-    gtk_widget_show (icon);
-    gtk_misc_set_alignment (GTK_MISC (icon), 0.5, 0);
-    gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+    data->theme_info_icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
+    data->theme_error_icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
+    gtk_misc_set_alignment (GTK_MISC (data->theme_info_icon), 0.5, 0);
+    gtk_misc_set_alignment (GTK_MISC (data->theme_error_icon), 0.5, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), data->theme_info_icon, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), data->theme_error_icon, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), data->theme_message_label, TRUE, TRUE, 0);
     gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (data->theme_message_area), hbox);
 
@@ -556,27 +706,64 @@ theme_message_area_update (AppearanceData *data)
     gtk_box_pack_start (GTK_BOX (parent), data->theme_message_area, FALSE, FALSE, 0);
   }
 
-  if (show_apply_background && show_apply_font) {
-    gtk_widget_show (data->theme_message_area);
-    gtk_widget_show (data->apply_background_button);
-    gtk_widget_show (data->apply_font_button);
+  if (show_error)
+    message = error->message;
+  else if (show_apply_background && show_apply_font && show_revert_font)
+    message = _("The current theme suggests a background and a font. Also, the last applied font suggestion can be reverted.");
+  else if (show_apply_background && show_revert_font)
+    message = _("The current theme suggests a background. Also, the last applied font suggestion can be reverted.");
+  else if (show_apply_background && show_apply_font)
     message = _("The current theme suggests a background and a font.");
-  } else if (show_apply_background) {
-    gtk_widget_show (data->theme_message_area);
-    gtk_widget_show (data->apply_background_button);
-    gtk_widget_hide (data->apply_font_button);
+  else if (show_apply_font && show_revert_font)
+    message = _("The current theme suggests a font. Also, the last applied font suggestion can be reverted.");
+  else if (show_apply_background)
     message = _("The current theme suggests a background.");
-  } else if (show_apply_font) {
-    gtk_widget_show (data->theme_message_area);
-    gtk_widget_hide (data->apply_background_button);
-    gtk_widget_show (data->apply_font_button);
+  else if (show_revert_font)
+    message = _("The last applied font suggestion can be reverted.");
+  else if (show_apply_font)
     message = _("The current theme suggests a font.");
+  else
+    message = NULL;
+
+  if (show_apply_background)
+    gtk_widget_show (data->apply_background_button);
+  else
+    gtk_widget_hide (data->apply_background_button);
+
+  if (show_apply_font)
+    gtk_widget_show (data->apply_font_button);
+  else
+    gtk_widget_hide (data->apply_font_button);
+
+  if (show_revert_font)
+    gtk_widget_show (data->revert_font_button);
+  else
+    gtk_widget_hide (data->revert_font_button);
+
+  if (show_error
+      && g_error_matches (error, GNOME_THEME_ERROR, GNOME_THEME_ERROR_GTK_ENGINE_NOT_AVAILABLE)
+      && packagekit_available ())
+    gtk_widget_show (data->install_button);
+  else
+    gtk_widget_hide (data->install_button);
+
+  if (show_error || show_apply_background || show_apply_font || show_revert_font) {
+    gtk_widget_show (data->theme_message_area);
+    gtk_widget_queue_draw (data->theme_message_area);
+
+    if (show_error) {
+      gtk_widget_show (data->theme_error_icon);
+      gtk_widget_hide (data->theme_info_icon);
+    } else {
+      gtk_widget_show (data->theme_info_icon);
+      gtk_widget_hide (data->theme_error_icon);
+    }
   } else {
     gtk_widget_hide (data->theme_message_area);
-    message = NULL;
   }
 
   gtk_label_set_text (GTK_LABEL (data->theme_message_label), message);
+  g_clear_error (&error);
 }
 
 static void
@@ -792,28 +979,22 @@ theme_drag_data_received_cb (GtkWidget *widget,
                              guint info, guint time,
                              AppearanceData *data)
 {
-  GList *uris;
+  gchar **uris;
 
   if (!(info == TARGET_URI_LIST || info == TARGET_NS_URL))
     return;
 
-  uris = gnome_vfs_uri_list_parse ((gchar *) selection_data->data);
+  uris = g_uri_list_extract_uris ((gchar *) selection_data->data);
 
-  if (uris != NULL && uris->data != NULL) {
-    GnomeVFSURI *uri = (GnomeVFSURI *) uris->data;
-    gchar *filename;
+  if (uris != NULL && uris[0] != NULL) {
+    GFile *f = g_file_new_for_uri (uris[0]);
 
-    if (gnome_vfs_uri_is_local (uri))
-      filename = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri), G_DIR_SEPARATOR_S);
-    else
-      filename = gnome_vfs_unescape_string (gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE), G_DIR_SEPARATOR_S);
-
-    gnome_vfs_uri_list_unref (uris);
-
-    gnome_theme_install_from_uri (filename,
+    gnome_theme_install (f,
         GTK_WINDOW (glade_xml_get_widget (data->xml, "appearance_window")));
-    g_free (filename);
+    g_object_unref (f);
   }
+
+  g_strfreev (uris);
 }
 
 static void
@@ -838,11 +1019,18 @@ themes_init (AppearanceData *data)
   GtkSettings *settings;
 
   /* initialise some stuff */
-  gnome_theme_init (NULL);
+  gnome_theme_init ();
   gnome_wm_manager_init ();
 
+  data->revert_application_font = NULL;
+  data->revert_documents_font = NULL;
+  data->revert_desktop_font = NULL;
+  data->revert_windowtitle_font = NULL;
+  data->revert_monospace_font = NULL;
   data->theme_save_dialog = NULL;
   data->theme_message_area = NULL;
+  data->theme_info_icon = NULL;
+  data->theme_error_icon = NULL;
   data->theme_custom = gnome_theme_meta_info_new ();
   data->theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/theme-thumbnailing.png", NULL);
   data->theme_store = theme_store =
@@ -896,7 +1084,7 @@ themes_init (AppearanceData *data)
   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (icon_view), renderer,
                                   "pixbuf", COL_THUMBNAIL, NULL);
 
-  renderer = gtk_cell_renderer_text_new ();
+  renderer = cell_renderer_caption_new ();
   g_object_set (renderer, "alignment", PANGO_ALIGN_CENTER,
 			  "wrap-mode", PANGO_WRAP_WORD_CHAR,
 			  "wrap-width", gtk_icon_view_get_item_width (icon_view),
@@ -959,6 +1147,19 @@ themes_init (AppearanceData *data)
   g_signal_connect (settings, "notify::gtk-color-scheme", (GCallback) theme_setting_changed_cb, data);
   g_signal_connect (settings, "notify::gtk-theme-name", (GCallback) theme_setting_changed_cb, data);
   g_signal_connect (settings, "notify::gtk-icon-theme-name", (GCallback) theme_setting_changed_cb, data);
+
+  /* monitor individual font choice buttons, so
+     "revert font" option (if any) can be cleared */
+  w = glade_xml_get_widget (data->xml, "application_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
+  w = glade_xml_get_widget (data->xml, "document_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
+  w = glade_xml_get_widget (data->xml, "desktop_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
+  w = glade_xml_get_widget (data->xml, "window_title_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
+  w = glade_xml_get_widget (data->xml, "monospace_font");
+  g_signal_connect (w, "font_set", (GCallback) custom_font_cb, data);
 }
 
 void
@@ -970,4 +1171,9 @@ themes_shutdown (AppearanceData *data)
     g_object_unref (data->theme_icon);
   if (data->theme_save_dialog)
     gtk_widget_destroy (data->theme_save_dialog);
+  g_free (data->revert_application_font);
+  g_free (data->revert_documents_font);
+  g_free (data->revert_desktop_font);
+  g_free (data->revert_windowtitle_font);
+  g_free (data->revert_monospace_font);
 }
