@@ -26,7 +26,9 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
+#include <glib/gstdio.h>
 
+#include "capplet-util.h"
 #include "file-transfer-dialog.h"
 #include "theme-installer.h"
 #include "theme-util.h"
@@ -48,17 +50,18 @@ enum {
 	DIRECTORY
 };
 
-static void
-cleanup_tmp_dir (const gchar *tmp_dir)
+static gboolean
+cleanup_tmp_dir (GIOSchedulerJob *job,
+		 GCancellable *cancellable,
+		 const gchar *tmp_dir)
 {
-	if (gnome_vfs_remove_directory (tmp_dir) == GNOME_VFS_ERROR_DIRECTORY_NOT_EMPTY) {
-		GList *list;
+	GFile *directory;
 
-		list = g_list_prepend (NULL, gnome_vfs_uri_new (tmp_dir));
-		gnome_vfs_xfer_delete_list (list, GNOME_VFS_XFER_RECURSIVE,
-					GNOME_VFS_XFER_ERROR_MODE_ABORT, NULL, NULL);
-		gnome_vfs_uri_list_free (list);
-	}
+	directory = g_file_new_for_path (tmp_dir);
+	capplet_file_delete_recursive (directory, NULL);
+	g_object_unref (directory);
+
+	return FALSE;
 }
 
 static int
@@ -74,16 +77,12 @@ file_theme_type (const gchar *dir)
 	exists = g_file_test (filename, G_FILE_TEST_IS_REGULAR);
 
 	if (exists) {
-		GPatternSpec *pattern = NULL;
+		GPatternSpec *pattern;
 		gchar *file_contents = NULL;
-		gint file_size;
-		gchar *uri;
+		gsize file_size;
 		gboolean match;
 
-		uri = gnome_vfs_get_uri_from_local_path (filename);
-		g_free (filename);
-		gnome_vfs_read_entire_file (uri, &file_size, &file_contents);
-		g_free (uri);
+		g_file_get_contents (filename, &file_contents, &file_size, NULL);
 
 		pattern = g_pattern_spec_new ("*[Icon Theme]*");
 		match = g_pattern_match_string (pattern, file_contents);
@@ -93,6 +92,7 @@ file_theme_type (const gchar *dir)
 			pattern = g_pattern_spec_new ("*Directories=*");
 			match = g_pattern_match_string (pattern, file_contents);
 			g_pattern_spec_free (pattern);
+			g_free (file_contents);
 
 			if (match) {
 				/* check if we have a cursor, too */
@@ -111,6 +111,7 @@ file_theme_type (const gchar *dir)
 		pattern = g_pattern_spec_new ("*[X-GNOME-Metatheme]*");
 		match = g_pattern_match_string (pattern, file_contents);
 		g_pattern_spec_free (pattern);
+		g_free (file_contents);
 
 		if (match)
 			return THEME_GNOME;
@@ -153,7 +154,12 @@ file_theme_type (const gchar *dir)
 static void
 transfer_cancel_cb (GtkWidget *dlg, gchar *path)
 {
-	gnome_vfs_unlink (path);
+	GFile *todelete;
+
+	todelete = g_file_new_for_path (path);
+	g_file_delete (todelete, NULL, NULL);
+
+	g_object_unref (todelete);
 	g_free (path);
 	gtk_widget_destroy (dlg);
 }
@@ -259,8 +265,8 @@ gnome_theme_install_real (gint filetype, const gchar *tmp_dir, const gchar *them
 {
 	gboolean success = TRUE;
 	GtkWidget *dialog, *apply_button;
-	int xfer_options;
-	GnomeVFSURI *theme_source_dir, *theme_dest_dir;
+	GFile *theme_source_dir, *theme_dest_dir;
+	GError *error = NULL;
 	gint theme_type;
 	gchar *user_message = NULL;
 	gchar *target_dir = NULL;
@@ -307,13 +313,25 @@ gnome_theme_install_real (gint filetype, const gchar *tmp_dir, const gchar *them
 		    && (file_theme_type (path) == THEME_ICON))
 		{
 			gchar *new_path, *update_icon_cache;
+			GFile *new_file;
+			GFile *src_file;
 
+			src_file = g_file_new_for_path (path);
 			new_path = g_build_path (G_DIR_SEPARATOR_S,
 						 g_get_home_dir (),
 						 ".icons",
 						 theme_name, NULL);
-			/* XXX: make some noise if we couldn't install it? */
-			gnome_vfs_move (path, new_path, FALSE);
+			new_file = g_file_new_for_path (new_path);
+
+			if (!g_file_move (src_file, new_file, G_FILE_COPY_NONE,
+					  NULL, NULL, NULL, &error)) {
+				g_warning ("Error while moving from `%s' to `%s': %s",
+					   path, new_path, error->message);
+				g_error_free (error);
+				error = NULL;
+			}
+			g_object_unref (new_file);
+			g_object_unref (src_file);
 
 			/* update icon cache - shouldn't really matter if this fails */
 			update_icon_cache = g_strdup_printf ("gtk-update-icon-cache %s", new_path);
@@ -326,17 +344,12 @@ gnome_theme_install_real (gint filetype, const gchar *tmp_dir, const gchar *them
 	}
 
 	/* Move the dir to the target dir */
-	theme_source_dir = gnome_vfs_uri_new (tmp_dir);
-	theme_dest_dir = gnome_vfs_uri_new (target_dir);
+	theme_source_dir = g_file_new_for_path (tmp_dir);
+	theme_dest_dir = g_file_new_for_path (target_dir);
 
-	xfer_options = GNOME_VFS_XFER_DELETE_ITEMS | GNOME_VFS_XFER_RECURSIVE;
-	if (filetype != DIRECTORY)
-		xfer_options |= GNOME_VFS_XFER_REMOVESOURCE;
-
-	if (gnome_vfs_xfer_uri (theme_source_dir, theme_dest_dir, xfer_options,
-				GNOME_VFS_XFER_ERROR_MODE_ABORT,
-				GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
-				NULL, NULL) != GNOME_VFS_OK) {
+	if (!g_file_move (theme_source_dir, theme_dest_dir,
+			  G_FILE_COPY_OVERWRITE, NULL, NULL,
+			  NULL, &error)) {
 		gchar *str;
 
 		str = g_strdup_printf (_("Installation for theme \"%s\" failed."), theme_name);
@@ -345,7 +358,13 @@ gnome_theme_install_real (gint filetype, const gchar *tmp_dir, const gchar *them
 					GTK_MESSAGE_ERROR,
 					GTK_BUTTONS_OK,
 					str);
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  error->message);
+
 		g_free (str);
+		g_error_free (error);
+		error = NULL;
+
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		success = FALSE;
@@ -475,12 +494,13 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 		gchar *tmp_dir;
 		gboolean ok;
 		gint n_themes;
+		GFile *todelete;
 
 		tmp_dir = g_strdup_printf ("%s/.themes/.theme-%u",
 					   g_get_home_dir (),
 					   g_random_int ());
 
-		if ((gnome_vfs_make_directory (tmp_dir, 0700)) != GNOME_VFS_OK) {
+		if ((g_mkdir (tmp_dir, 0700)) != 0) {
 			dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
 							GTK_BUTTONS_OK,
 							_("Failed to create temporary directory"));
@@ -491,17 +511,20 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 			return;
 		}
 
-
 		if (!transfer_done_archive (filetype, tmp_dir, path) ||
 		    ((dir = g_dir_open (tmp_dir, 0, NULL)) == NULL))
 		{
-			cleanup_tmp_dir (tmp_dir);
+			g_io_scheduler_push_job ((GIOSchedulerJobFunc) cleanup_tmp_dir,
+						 g_strdup (tmp_dir), g_free,
+						 G_PRIORITY_DEFAULT, NULL);
 			g_free (tmp_dir);
 			g_free (path);
 			return;
 		}
 
-		gnome_vfs_unlink (path);
+		todelete = g_file_new_for_path (path);
+		g_file_delete (todelete, NULL, NULL);
+		g_object_unref (todelete);
 
 		/* See whether we have multiple themes to install. If so,
 		 * we won't ask the user whether to apply the new theme
@@ -547,25 +570,23 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
 		}
-
-		cleanup_tmp_dir (tmp_dir);
-		g_free (tmp_dir);
+		g_io_scheduler_push_job ((GIOSchedulerJobFunc) cleanup_tmp_dir,
+					 tmp_dir, g_free,
+					 G_PRIORITY_DEFAULT, NULL);
 	}
 
 	g_free (path);
 }
 
 void
-gnome_theme_install_from_uri (const gchar *filename, GtkWindow *parent)
+gnome_theme_install (GFile *file, GtkWindow *parent)
 {
 	GtkWidget *dialog;
 	gchar *path, *base;
 	GList *src, *target;
-	gchar *temppath;
 	const gchar *template;
-	int cmp;
 
-	if (filename == NULL || strcmp (filename, "") == 0) {
+	if (file == NULL) {
 		dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_MODAL,
 						 GTK_MESSAGE_ERROR,
@@ -576,9 +597,10 @@ gnome_theme_install_from_uri (const gchar *filename, GtkWindow *parent)
 		return;
 	}
 
-	/* see if someone dropped a directory */
-	if (g_file_test (filename, G_FILE_TEST_IS_DIR))	{
-		transfer_done_cb (NULL, g_strdup (filename));
+	/* see if someone dropped a local directory */
+	if (g_file_is_native (file) &&
+	    g_file_query_file_type (file, G_FILE_QUERY_INFO_NONE, NULL) == G_FILE_TYPE_DIRECTORY) {
+		transfer_done_cb (NULL, g_file_get_path (file));
 		return;
 	}
 
@@ -597,28 +619,7 @@ gnome_theme_install_from_uri (const gchar *filename, GtkWindow *parent)
 		return;
 	}
 
-	/* To avoid the copy of /root/.themes to /root/.themes/.themes
-	 * which causes an infinite loop. The user asks to transfer the all
-	 * contents of a folder, to a folder under itself. So ignore the
-	 * situation.
-	 */
-	temppath = g_build_filename (filename, ".themes", NULL);
-	cmp = strcmp (temppath, path);
-	g_free (path);
-	g_free (temppath);
-
-	if (cmp == 0) {
-		dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						 _("%s is the path where the theme files will be installed. This can not be selected as the source location"), filename);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-		return;
-	}
-
-	base = g_path_get_basename (filename);
+	base = g_file_get_basename (file);
 
 	if (g_str_has_suffix (base, ".tar.gz") || g_str_has_suffix (base, ".tgz") || g_str_has_suffix (base, ".gtp"))
 		template = "gnome-theme-%d.gtp";
@@ -627,12 +628,11 @@ gnome_theme_install_from_uri (const gchar *filename, GtkWindow *parent)
 	else {
 		invalid_theme_dialog (base, FALSE);
 		g_free (base);
-		gnome_vfs_uri_list_unref (src);
 		return;
 	}
 	g_free (base);
 
-	src = g_list_append (NULL, g_strdup (filename));
+	src = g_list_append (NULL, g_object_ref (file));
 
 	path = NULL;
 	do {
@@ -645,7 +645,7 @@ gnome_theme_install_from_uri (const gchar *filename, GtkWindow *parent)
 	} while (g_file_test (path, G_FILE_TEST_EXISTS));
 
 
-	target = g_list_append (NULL, path);
+	target = g_list_append (NULL, g_file_new_for_path (path));
 
 	dialog = file_transfer_dialog_new_with_parent (parent);
 	g_signal_connect (dialog, "cancel",
@@ -655,12 +655,14 @@ gnome_theme_install_from_uri (const gchar *filename, GtkWindow *parent)
 
 	file_transfer_dialog_copy_async (FILE_TRANSFER_DIALOG (dialog),
 					 src, target,
+					 FILE_TRANSFER_DIALOG_DEFAULT,
 					 G_PRIORITY_DEFAULT);
 	gtk_widget_show (dialog);
 
-	g_list_foreach (src, (GFunc) g_free, NULL);
-	/* don't free the target item since we're using that for the signals */
+	/* don't free the path since we're using that for the signals */
+	g_list_foreach (src, (GFunc) g_object_unref, NULL);
 	g_list_free (src);
+	g_list_foreach (target, (GFunc) g_object_unref, NULL);
 	g_list_free (target);
 }
 
@@ -670,7 +672,6 @@ gnome_theme_installer_run (GtkWindow *parent, const gchar *filename)
 	static gboolean running_theme_install = FALSE;
 	static gchar old_folder[512] = "";
 	GtkWidget *dialog;
-	gchar *filename_selected, *folder;
 	GtkFileFilter *filter;
 
 	if (running_theme_install)
@@ -701,7 +702,9 @@ gnome_theme_installer_run (GtkWindow *parent, const gchar *filename)
 
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
 	{
-		filename_selected = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+		gchar *uri_selected, *folder;
+
+		uri_selected = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
 
 		folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog));
 		g_strlcpy (old_folder, folder, 255);
@@ -709,8 +712,13 @@ gnome_theme_installer_run (GtkWindow *parent, const gchar *filename)
 
 		gtk_widget_destroy (dialog);
 
-		gnome_theme_install_from_uri (filename_selected, parent);
-		g_free (filename_selected);
+		if (uri_selected != NULL) {
+			GFile *file = g_file_new_for_uri (uri_selected);
+			g_free (uri_selected);
+
+			gnome_theme_install (file, parent);
+			g_object_unref (file);
+		}
 	}
 	else
 		gtk_widget_destroy (dialog);
